<html>

<head>
    <style>
    li {
        padding-bottom: 4px;
    }
    </style>
</head>

<body>
    <h1>Perimeter Gateway</h1>
    <p>In order to access this site you will need a valid <strong>access token</strong>.</p>
    <form action="" method="post">
        {% csrf_token %}
        {{ form }}
        <input type="submit" value="Submit">
    </form>
    <p><small>Your browser user-agent and IP address will be recorded when you validate your token.</small></p>
</body>

</html>
